/*******************************************************************************
 *                                                                             *
 *   F1LT - unofficial Formula 1 live timing application                       *
 *   Copyright (C) 2012-2013  Mariusz Pilarek (pieczaro@gmail.com)             *
 *                                                                             *
 *   This program is free software: you can redistribute it and/or modify      *
 *   it under the terms of the GNU General Public License as published by      *
 *   the Free Software Foundation, either version 3 of the License, or         *
 *   (at your option) any later version.                                       *
 *                                                                             *
 *   This program is distributed in the hope that it will be useful,           *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *   GNU General Public License for more details.                              *
 *                                                                             *
 *   You should have received a copy of the GNU General Public License         *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
 *                                                                             *
 *******************************************************************************/

#include "driverradararea.h"
#include "../../core/eventdata.h"
#include "../../core/colorsmanager.h"

namespace DriverTrackerTool
{

DriverRadarArea::DriverRadarArea(QGraphicsItem *parent) :
    QGraphicsObject(parent), selectedDriver(-2)
{
    radarX = 0;
    radarY = 0;
    radarR = 50;
    radarLappedR = radarR * 0.75;
    radarPitR = radarR * 0.5;
    setCacheMode(QGraphicsItem::DeviceCoordinateCache);
}

QRectF DriverRadarArea::boundingRect() const
{
    qreal w = (radarR + 40) * 2;
    qreal h = w;

    return QRectF(-w/2.0, -h/2.0, w, h);
}

void DriverRadarArea::paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *)
{
    painter->setRenderHint(QPainter::Antialiasing);

    QPen pen(QColor(255, 255, 255), 5);

    if (EventData::getInstance().getFlagStatus() == LTPackets::SAFETY_CAR_DEPLOYED)
        pen.setColor(ColorsManager::getInstance().getDefaultColor(LTPackets::YELLOW));

    painter->setBrush(QBrush());
    painter->setPen(pen);
    painter->drawEllipse(QPoint(radarX, radarY), (int)radarR, (int)radarR);

    pen.setWidth(3);
    painter->setPen(pen);
    painter->drawEllipse(QPoint(radarX, radarY), (int)radarLappedR, (int)radarLappedR);

    pen.setColor(QColor(255, 255, 255));
    pen.setWidth(5);
    painter->setPen(pen);

    int x = radarX - radarPitR;
    int y = radarY - radarPitR;
    int w = radarX + radarPitR - x;
    int h = radarY + radarPitR - y;
    painter->drawArc(x, y, w, h, 270*16, 180*16);

    pen.setWidth(2);
    painter->setPen(pen);
    painter->drawLine(radarX, radarY - radarR - 10, radarX, radarY - radarLappedR + 10);
}

void DriverRadarArea::checkSetupCorrect(int speed)
{
    if (drp.isEmpty() || (selectedDriver == -2))
    {
        setup(speed);
    }
    else
    {
        for (int i = 0; i < drp.size(); ++i)
        {
            drp[i]->setSpeed(speed);
        }
    }
}

void DriverRadarArea::loadDriversList()
{
    selectedDriver = -1;
    for (int i = 0; i < drp.size(); ++i)
        delete drp[i];

    drp.resize(EventData::getInstance().getDriversData().size());

    for (int i = drp.size()-1; i >= 0; --i)
    {
        drp[i] = new DriverRadarPositioner(this, &EventData::getInstance().getDriversData()[i]);
        connect (drp[i], SIGNAL(driverSelected(DriverRadarPositioner*)), this, SLOT(onDRDriverSelected(DriverRadarPositioner*)), Qt::UniqueConnection);
    }
}

void DriverRadarArea::updateSize(int width)
{
    radarR = width / 2 - 40;
    radarLappedR = radarR * 0.75;
    radarPitR = radarR * 0.5;

    update();

    for (int i = 0; i < drp.size(); ++i)
        drp[i]->setRadarCoords(radarX, radarY, radarR, radarPitR, radarLappedR);
}


void DriverRadarArea::setup(int speed)
{
    if (selectedDriver != -2)
        selectedDriver = -1;

    radarX = x();
    radarY = y();

    for (int i = 0; i < drp.size(); ++i)
    {
        drp[i]->setRadarCoords(radarX, radarY, radarR, radarPitR, radarLappedR);
        drp[i]->setStartupPosition();
        drp[i]->setExcluded(false);
        drp[i]->setSpeed(speed);
    }
}

void DriverRadarArea::updatePosition()
{
    if (selectedDriver == -2)
        selectedDriver = -1;

    for (int i = 0; i < drp.size(); ++i)
        drp[i]->updatePosition();

}

void DriverRadarArea::selectDriver(int id)
{
    selectedDriver = id;
    for (int i = 0; i < drp.size(); ++i)
    {
        drp[i]->setSelected(false);

        if (id == drp[i]->getDriverId())
        {
            selectedDriver = id;
            drp[i]->setSelected(true);
        }
    }
}

void DriverRadarArea::excludeDriver(int id, bool exclude)
{
    for (int i = 0; i < drp.size(); ++i)
    {
        if (drp[i]->getDriverId() == id)
        {
            drp[i]->setExcluded(exclude);
        }
    }
}

void DriverRadarArea::onDRDriverSelected(DriverRadarPositioner *driver)
{
    if (driver)
    {
        /* A single press can be detected by several drivers (if they are close to each other), but only one can be selected
         * at a time. So all others have to be deselected then
         */
        for (int i = 0; i < drp.size(); ++i)
        {
            if (drp[i] != driver)
            {
                drp[i]->setSelected(false);
            }
        }
        selectedDriver = driver->getDriverId();
        emit driverSelected(driver->getDriverId());
    }
    else
    {
        selectedDriver = -1;
        emit driverSelected(-1);
    }
}

}
