/*******************************************************************************
 *                                                                             *
 *   F1LT - unofficial Formula 1 live timing application                       *
 *   Copyright (C) 2012-2013  Mariusz Pilarek (pieczaro@gmail.com)             *
 *                                                                             *
 *   This program is free software: you can redistribute it and/or modify      *
 *   it under the terms of the GNU General Public License as published by      *
 *   the Free Software Foundation, either version 3 of the License, or         *
 *   (at your option) any later version.                                       *
 *                                                                             *
 *   This program is distributed in the hope that it will be useful,           *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *   GNU General Public License for more details.                              *
 *                                                                             *
 *   You should have received a copy of the GNU General Public License         *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
 *                                                                             *
 *******************************************************************************/

#include "drivertrackerarea.h"
#include <QPainter>
#include <QDebug>

#include "../../core/eventdata.h"

#include <QGraphicsPixmapItem>

namespace DriverTrackerTool
{

DriverTrackerArea::DriverTrackerArea(QGraphicsItem *parent) :
    QGraphicsObject(parent), selectedDriver(-2)
{
    setCacheMode(QGraphicsItem::DeviceCoordinateCache) ;
}

QRectF DriverTrackerArea::boundingRect() const
{
    int w = trackMap.width();
    int h = trackMap.height();

    return QRectF(-w/2.0, -h/2.0, w, h);
}

void DriverTrackerArea::paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *)
{
    if (!trackMap.isNull())
    {
        int w = trackMap.width();
        int h = trackMap.height();

        painter->setRenderHint(QPainter::SmoothPixmapTransform);
        painter->drawPixmap(- w/2, - h/2, trackMap);
    }
}

void DriverTrackerArea::checkSetupCorrect(int speed)
{
    trackMap = EventData::getInstance().getEventInfo().trackImg;

    if (dtp.isEmpty() || (selectedDriver == -2))
    {
        setup(speed);
    }
    else
    {
        for (int i = 0; i < dtp.size(); ++i)
        {
            dtp[i]->setSpeed(speed);
            dtp[i]->updateScaleAndRotation();
        }
    }
}

void DriverTrackerArea::loadDriversList()
{
    selectedDriver = -1;
    for (int i = 0; i < dtp.size(); ++i)
        delete dtp[i];

    dtp.resize(EventData::getInstance().getDriversData().size());

    //drivers are adding in reverse order, so that the leaders will be always on top
    for (int i = dtp.size()-1; i >= 0; --i)
    {
        dtp[i] = new DriverTrackerPositioner(this, &EventData::getInstance().getDriversData()[i]);
        connect (dtp[i], SIGNAL(driverSelected(DriverTrackerPositioner*)), this, SLOT(onDTPDriverSelected(DriverTrackerPositioner*)), Qt::UniqueConnection);
    }
}

DriverTrackerPositioner *DriverTrackerArea::getSelectedDriver()
{
    if (selectedDriver == -1)
        return NULL;

    for (int i = 0; i < dtp.size(); ++i)
    {
        if (dtp[i]->getDriverId() == selectedDriver)
            return dtp[i];
    }

    return NULL;
}

void DriverTrackerArea::setup(int speed)
{
    trackMap = EventData::getInstance().getEventInfo().trackImg;

    if (selectedDriver != -2)
        selectedDriver = -1;

    for (int i = 0; i < dtp.size(); ++i)
    {
        dtp[i]->setMapCoords(scenePos().x(), scenePos().y(), trackMap.width(), trackMap.height());
        dtp[i]->setStartupPosition();
        dtp[i]->setExcluded(false);
        dtp[i]->setSpeed(speed);
    }
}

void DriverTrackerArea::updatePosition()
{
    if (selectedDriver == -2)
        selectedDriver = -1;

    for (int i = 0; i < dtp.size(); ++i)
        dtp[i]->updatePosition();

}

void DriverTrackerArea::onDTPDriverSelected(DriverTrackerPositioner *driver)
{
    if (driver)
    {
        /* A single press can be detected by several drivers (if they are close to each other), but only one can be selected
         * at a time. So all others have to be deselected then
         */
        for (int i = 0; i < dtp.size(); ++i)
        {
            if (dtp[i] != driver)
            {
                dtp[i]->setSelected(false);
            }
        }


        selectedDriver = driver->getDriverId();
    }
    else
    {
        selectedDriver = -1;
    }
    emit driverSelected(selectedDriver);
}

void DriverTrackerArea::selectDriver(int id)
{
    selectedDriver = id;

    for (int i = 0; i < dtp.size(); ++i)
    {
        dtp[i]->setSelected(false);

        if ((id > 0) && (id == dtp[i]->getDriverId()))
        {
            dtp[i]->setSelected(true);
        }
    }
}

void DriverTrackerArea::excludeDriver(int id, bool exclude)
{
    for (int i = 0; i < dtp.size(); ++i)
    {
        if (dtp[i]->getDriverId() == id)
        {
            dtp[i]->setExcluded(exclude);
        }
    }
}

void DriverTrackerArea::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    /* This function checks whether a mouse button has been pressed somewhere on the track, but not on any driver.
     * If yes - all drivers have to be deselected and appropriate signal has to be emitted to inform driverTrackerWidget that
     * there is no driver to follow
     */
    if (event->button() == Qt::LeftButton)
    {
        bool driverFound = false;
        for (int i = 0; i < dtp.size(); ++i)
        {
            if (!dtp[i]->contains(event->pos()))
                dtp[i]->setSelected(false);
            else
                driverFound = true;
        }

        if (!driverFound)
            emit driverSelected(-1);
    }
}




}
