/*******************************************************************************
 *                                                                             *
 *   F1LT - unofficial Formula 1 live timing application                       *
 *   Copyright (C) 2012-2013  Mariusz Pilarek (pieczaro@gmail.com)             *
 *                                                                             *
 *   This program is free software: you can redistribute it and/or modify      *
 *   it under the terms of the GNU General Public License as published by      *
 *   the Free Software Foundation, either version 3 of the License, or         *
 *   (at your option) any later version.                                       *
 *                                                                             *
 *   This program is distributed in the hope that it will be useful,           *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *   GNU General Public License for more details.                              *
 *                                                                             *
 *   You should have received a copy of the GNU General Public License         *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
 *                                                                             *
 *******************************************************************************/


#include "drivertrackerpositioner.h"

#include "../../core/colorsmanager.h"
#include "../../core/eventdata.h"

#include <QStyleOptionGraphicsItem>

namespace DriverTrackerTool
{

DriverTrackerPositioner::DriverTrackerPositioner(QGraphicsItem *parent, DriverData *dd) : DriverRadarPositioner(parent, dd), coordinatesCount(0), mapX(0), mapY(0)
{
    viewRotation = 0;
    viewScale = 1.0;
    selected = false;

    setCacheMode(QGraphicsItem::DeviceCoordinateCache);
}



void DriverTrackerPositioner::setStartupPosition()
{
    int laps = EventData::getInstance().getEventInfo().laps;
    lapped = false;
    qualiOut = false;
    inPits = false;
    finished = false;

    coordinatesCount = SeasonData::getInstance().getTrackMapsCoordinates().getCurrentTrackCoordinates().coordinates.size();//EventData::getInstance().getEventInfo().trackCoordinates.coordinates.size();

    //a very rough estimate ;)
    avgTime = 200 - 30 * log10(laps*laps);

    if (EventData::getInstance().getSessionRecords().getFastestLap().getTime().isValid())
        avgTime = EventData::getInstance().getSessionRecords().getFastestLap().getTime().toDouble();

    //if it's raining, we add 10 seconds
    if (EventData::getInstance().getWeather().getWetDry().getValue() == 1)
        avgTime += 10;

    //if SC is on track, we add 1 minute
    if (EventData::getInstance().getFlagStatus() == LTPackets::SAFETY_CAR_DEPLOYED)
        avgTime += 60;

    if (driverData && (driverData->isInPits() || driverData->isRetired()))
    {
        goToPit(true);
        calculatePitPosition();
    }

    else if (EventData::getInstance().getEventType() == LTPackets::RACE_EVENT &&
        (!EventData::getInstance().isSessionStarted() || !driverData->getLastLap().getSectorTime(1).isValid()))
    {
        goToPit(false);
        currentDeg = coordinatesCount-(round((double)driverData->getPosition()/3))-1;
        currentLapTime = -(avgTime-(avgTime * currentDeg) / coordinatesCount);
    }
    else
    {
        currentLapTime = 0;
        for (int i = 0; i < 3; ++i)
            currentLapTime += driverData->getLastLap().getSectorTime(i+1).toDouble();

        calculatePosition();
    }
}


void DriverTrackerPositioner::calculatePosition()
{
    EventData &ev = EventData::getInstance();
    if (!ev.isSessionStarted())
        return;

    TrackCoordinates trackCoordinates = SeasonData::getInstance().getTrackMapsCoordinates().getCurrentTrackCoordinates();
    if (/*driverData->getLastLap().getTime().toString() != "OUT" &&*/ currentDeg > 0)
    {
        if (driverData->getLastLap().getSectorTime(1).isValid() &&
                ((ev.getEventType() == LTPackets::RACE_EVENT && driverData->getColorData().lapTimeColor() == LTPackets::YELLOW) ||
                 (!driverData->getLastLap().getSectorTime(2).isValid() || !driverData->getLastLap().getSectorTime(3).isValid())) &&
                currSector == 1)
        {
            currSector = 2;
            currentDeg = trackCoordinates.indexes[0];
        }
        else if (driverData->getLastLap().getSectorTime(2).isValid() &&
                 !driverData->getLastLap().getSectorTime(3).isValid() && currSector == 2 &&
                 driverData->getLastLap().getTime().toString() != "OUT")
        {
            currSector = 3;
            currentDeg = trackCoordinates.indexes[1];
        }
        else
        {
            currentDeg += ((double)coordinatesCount / avgTime) / speed; //(360 * currentLapTime) / avgTime;
        }
    }
    else
    {
        currentDeg += ((double)coordinatesCount / avgTime) / speed;
    }

    if ((int)(round(currentDeg)) >= coordinatesCount)
        currentDeg = 0;
}

void DriverTrackerPositioner::calculatePitOutPosition()
{
    currentDeg = SeasonData::getInstance().getTrackMapsCoordinates().getCurrentTrackCoordinates().indexes[2];
}

void DriverTrackerPositioner::rescaleLabels()
{
    int size = 12;

    if (viewScale >= 2.0)
    {
        size = 24;
    }
    else if (viewScale > 1.0)
    {
        size = 12 * viewScale;
    }

    label[NORMAL] = QPixmap(":/ui_icons/label-driver.png").scaledToHeight(size, Qt::SmoothTransformation);
    label[SELECTED] = QPixmap(":/ui_icons/label-driver-sel.png").scaledToHeight(size, Qt::SmoothTransformation);
    label[LEADER] = QPixmap(":/ui_icons/label-driver-leader.png").scaledToHeight(size, Qt::SmoothTransformation);
    label[RETIRED] = QPixmap(":/ui_icons/label-driver-retired.png").scaledToHeight(size, Qt::SmoothTransformation);
}

void DriverTrackerPositioner::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    if (event->button() == Qt::LeftButton)
    {
        if (!selected)
        {
            selected = true;
            emit driverSelected(this);
            setZValue(2);
            update();
        }
        else
        {
            selected = false;
            setZValue(1);
            emit driverSelected(NULL);
            update();
        }
    }
}

void DriverTrackerPositioner::setTrackPosition()
{
    if (inPits)
    {        
//        setVisible(false);
//        doUpdate = true;
    }

    if (coordinatesCount > 0)
    {
        if (!isVisible() && !excluded && !inPits)
        {
            update();
            setVisible(true);
        }

        int idx = (int)floor(currentDeg);//(int)round(currentDeg);
        double factor = currentDeg - (double)idx;

        if (idx >= (coordinatesCount - 1))
            idx = coordinatesCount - 2;

        double newX = mapX -(mapX + mapW /2) + SeasonData::getInstance().getTrackMapsCoordinates().getCurrentTrackCoordinates().coordinates[idx].x();
        double newY = -(mapY + mapH /2) + SeasonData::getInstance().getTrackMapsCoordinates().getCurrentTrackCoordinates().coordinates[idx].y();

        double nextX = mapX -(mapX + mapW /2) + SeasonData::getInstance().getTrackMapsCoordinates().getCurrentTrackCoordinates().coordinates[idx+1].x();
        double nextY = -(mapY + mapH /2) + SeasonData::getInstance().getTrackMapsCoordinates().getCurrentTrackCoordinates().coordinates[idx+1].y();

        double resX = newX + factor * (nextX - newX);
        double resY = newY + factor * (nextY - newY);

        if (resX != x() || resY != y())
            setPos(resX, resY);
    }
    else
    {
        setVisible(false);
//        doUpdate = true;
    }
}

void DriverTrackerPositioner::updatePosition()
{
    DriverRadarPositioner::updatePosition();
//    update();
//    setTrackPosition();
}

QRectF DriverTrackerPositioner::boundingRect() const
{
    if (driverData && driverData->getCarID() > 0)
    {
        int size = 24;

        if (viewScale >= 2.0)
        {
            size = 48;
        }
        else if (viewScale > 1.0)
        {
            size = 24 * viewScale;
        }
        QPixmap helmet = ImagesFactory::getInstance().getHelmetsFactory().getHelmet(driverData->getNumber(), size);

        int w = helmet.width() > label[0].width() ? helmet.width() : label[0].width();

        return QRectF(-w/2 - 8, -helmet.height()/2, w  + 16, helmet.height() + 4 + label[0].height());
    }
    return QRectF();
}

void DriverTrackerPositioner::paint(QPainter *p, const QStyleOptionGraphicsItem *option, QWidget *)
{
    if (driverData && driverData->getCarID() > 0 && !driverData->isInPits() && !driverData->isRetired() && !excluded)
    {
        p->setClipRect( option->exposedRect );
        setTrackPosition();


        QColor drvColor = ColorsManager::getInstance().getCarColor(driverData->getNumber());
        p->setBrush(QBrush(drvColor));

        QPen namePen(QPen(ColorsManager::getInstance().getColor(LTPackets::WHITE)));
        QPen rectPen(drvColor);

        QPixmap drvLabel = label[NORMAL];

        if (driverData->getPosition() == 1)
        {
            drvLabel = label[LEADER];
            namePen = QPen(ColorsManager::getInstance().getColor(LTPackets::BACKGROUND));
        }
        if (driverData->isRetired() || qualiOut)
        {
            drvLabel = label[RETIRED];
            namePen = QPen(ColorsManager::getInstance().getColor(LTPackets::BACKGROUND));
        }
        if (selected)
        {
            drvLabel = label[SELECTED];
            namePen = QPen(ColorsManager::getInstance().getColor(LTPackets::BACKGROUND));
        }

        int size = 24;

        if (viewScale >= 2.0)
        {
            size = 48;
        }
        else if (viewScale > 1.0)
        {
            size = 24 * viewScale;
        }

        p->setFont(QFont("Arial", size / 3, 75));

        QPixmap helmet = ImagesFactory::getInstance().getHelmetsFactory().getHelmet(driverData->getNumber(), size);
        p->drawPixmap(-size/2, boundingRect().y(), helmet);

        p->setPen(rectPen);
        int labelX = -drvLabel.width() / 2;
        int labelY = boundingRect().y() + helmet.height() + 4;

        int labelSize = size / 2;
        const QPointF points[4] = {
             QPointF(labelX + 2.0 + labelSize/2, labelY + 1.0),
             QPointF(labelX + labelSize + 8.0, labelY + 1.0),
             QPointF(labelX + labelSize + 8.0, labelY + drvLabel.height() - 2.0),
             QPointF(labelX + 1.0, labelY + drvLabel.height() - 2.0)
         };


        p->drawConvexPolygon(points, 4);
        p->drawPixmap(-drvLabel.width()/2, labelY, drvLabel);

        QString name = SeasonData::getInstance().getDriverShortName(driverData->getDriverName());//QString::number(driverData->getNumber());

        p->setPen(namePen);

        int numX = labelX + labelSize/3 + (drvLabel.width() - p->fontMetrics().width(name))/2;
        int numY = labelY + (drvLabel.height() + p->fontMetrics().height())/2 - 2;
        p->drawText(numX, numY, name);
    }
}

}
